package com.menagerie.ophelia.view

import androidx.compose.runtime.Composable
import androidx.compose.ui.res.stringResource
import androidx.lifecycle.viewmodel.compose.viewModel
import androidx.navigation.NavController
import androidx.navigation.NavGraphBuilder
import androidx.navigation.NavHostController
import androidx.navigation.NavType
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import androidx.navigation.navArgument
import com.menagerie.ophelia.R
import com.menagerie.ophelia.database.polycule.PolyculeDatabaseManager
import com.menagerie.ophelia.database.polycule.entity.viewmodel.BioListViewModel
import com.menagerie.ophelia.view.biographies.AddBiography
import com.menagerie.ophelia.view.biographies.BioDetailsScreen
import com.menagerie.ophelia.view.newUser.AddNameScreen
import com.menagerie.ophelia.view.newUser.AddPronounsScreen
import com.menagerie.ophelia.view.polycule.PolyculeHomeView

@Composable
fun PolyculeApp() {
    val navController = rememberNavController()
    PolyculeNavHost(
        navController = navController
    )
}

@Composable
fun PolyculeNavHost(
    navController: NavHostController
) {
    val home = stringResource(id = R.string.route_home)
    NavHost(
        navController = navController,
        startDestination = "home"
    ) {
        polyculeGraph(navController)
        welcomeGraph(navController)
        composable(home) {
            HomeScreen(!PolyculeDatabaseManager.hasPolycule()) {
                navController.navigate("newUserStart")
            }
        }

    }
}

fun NavGraphBuilder.welcomeGraph(navController: NavHostController) {
    composable("newUserStart") { NewUserStartScreen(){
        navController.navigate("newUserName")
    } }
    composable("newUserName") { AddNameScreen(
       onGo={ navController.navigate("newUserPronouns")}
    )}
    composable("newUserPronouns") {
        AddPronounsScreen()
    }
}

fun NavGraphBuilder.polyculeGraph(navController: NavController) {
    composable("polyculeHome") {
        PolyculeHomeView(
            onBioClick = {
                navController.navigate("bioDetail/${it.id}")
            },
            onAddClick = {
                navController.navigate("addBio"){}
            }
        )
    }
    composable(
        "bioDetail/{bioId}",
        arguments = listOf(navArgument("bioId") {
            type = NavType.IntType
        })
    ) {
        BioDetailsScreen(
            id = it.arguments?.getInt("bioId") ?: 0,
            onBackClick = { navController.navigateUp() },
        )
    }
    composable("addBio") {
        AddBiography()
    }
}