package com.menagerie.ophelia.view.biographies

import android.annotation.SuppressLint
import android.util.Log
import android.widget.Toast
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.text.KeyboardActions
import androidx.compose.material3.Scaffold
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.livedata.observeAsState
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.platform.LocalFocusManager
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewmodel.compose.viewModel
import com.menagerie.ophelia.R
import com.menagerie.ophelia.database.polycule.entity.Bio
import com.menagerie.ophelia.database.polycule.entity.viewmodel.BioListViewModel
import com.menagerie.ophelia.view.components.InputFieldComponent
import com.menagerie.ophelia.view.components.fab.FABComponent

fun insertBioInDB(bio: Bio?, mBioListViewModel: BioListViewModel) {
    bio?.let {
       mBioListViewModel.upsert(it)
    }
}

@SuppressLint("UnusedMaterial3ScaffoldPaddingParameter")
@Composable
fun AddBiography() {
    val inputViewModel = InputViewModel()
    val context = LocalContext.current
    val mBioListViewModel: BioListViewModel = viewModel(
        factory = BioListViewModel.BioListViewModelFactory()
    )

    Scaffold(
        floatingActionButton = {
            FABComponent(text = "${stringResource(id = R.string.add_bio)}", onClick = {
                insertBioInDB(inputViewModel.bio.value, mBioListViewModel)

                Toast.makeText(context, "Added Bio", Toast.LENGTH_SHORT).show()
            })
        }
    )
    {
        Column {
            val modifier = Modifier.width(120.dp)
            val bio: Bio by inputViewModel.bio.observeAsState(Bio())

            InputFieldState(inputViewModel, bio.name, "name", "Ophelia", modifier = Modifier) //Name
            Row {
                InputFieldState(inputViewModel, bio.subjective,"", "she", modifier)
                InputFieldState(inputViewModel, bio.objective, "", "her", modifier)
            }
            Row {
                InputFieldState(inputViewModel, bio.possessive,"", "hers", modifier)
                InputFieldState(inputViewModel, bio.reflexive, "","herself", modifier)
            }

        }
    }
}

@Composable
fun InputFieldState(
    inputViewModel: InputViewModel,
    value: String,
    label: String,
    placeholder: String,
    modifier: Modifier
) {

    Column(
        modifier = Modifier
    ) {
        InputField(value, label, placeholder, modifier) { inputViewModel.onInputChange(it) }
        Spacer(modifier = Modifier.padding(10.dp))
    }
}

@Composable
fun InputField(
    text: String,
    label: String,
    placeholder: String,
    modifier: Modifier,
    onValChange: ((String) -> Unit)?
) {
    val focusManager = LocalFocusManager.current

    if (onValChange != null) {
        InputFieldComponent(
            text = text,
            onChange = onValChange,
            label = label,
            placeHolder = placeholder,
            modifier = modifier,
            keyboardActions = KeyboardActions(onDone = { focusManager.clearFocus() })
        )
    }
}

class InputViewModel : ViewModel() {
    private val _bio: MutableLiveData<Bio> = MutableLiveData(Bio())
    val bio: LiveData<Bio> = _bio
    fun onInputChange(name: String) {
        Log.d("Tawni", name)
        val newBio = Bio(
            name = name,
        )
        _bio.value = newBio
    }
}

@Preview
@Composable
fun AddEditBioPreview()
{
    AddBiography()
}
