package com.menagerie.ophelia.view.biographies

import android.util.Log
import androidx.compose.foundation.Image
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Surface
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.livedata.observeAsState
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.platform.LocalDensity
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.unit.Dp
import androidx.compose.ui.unit.dp
import androidx.constraintlayout.compose.ConstraintLayout
import androidx.lifecycle.viewmodel.compose.viewModel
import com.menagerie.ophelia.database.polycule.entity.Bio
import com.menagerie.ophelia.database.polycule.entity.viewmodel.BioDetailViewModel
import com.menagerie.ophelia.view.components.utils.Dimens

data class BioDetailCallbacks(
    val onBackClick: () -> Unit
)

@Composable
fun BioDetailsScreen(
    id: Int,
    onBackClick: () -> Unit,
) {
    val mBioDetailViewModel: BioDetailViewModel = viewModel(
    )

    mBioDetailViewModel.setBio(bioId = id)

    val bio = mBioDetailViewModel.bio.observeAsState().value

    if (bio != null) {
        Surface {
            BioDetails(
                bio,
                BioDetailCallbacks(
                    onBackClick = onBackClick
                )
            )
        }
    }
}

@Composable
fun BioDetails(
    bio: Bio,
    callbacks: BioDetailCallbacks,
    modifier: Modifier = Modifier
) {
    Box(
        modifier
            .fillMaxSize()
    ) {
        BioDetailContents(
            bio = bio,
            imageHeight = with(LocalDensity.current) {
                1.dp
            },
        )
    }
}

@Composable
fun BioDetailContents(
    bio: Bio,
    imageHeight: Dp,
) {
    Column {
        ConstraintLayout {
            val (image, info) = createRefs()
            Log.d("Tawni" ,"${bio.id}")
            Image(
                painter = painterResource(id = bio.pfpRes),
                contentDescription = null,
                modifier = Modifier
                    .size(130.dp)
                    .padding(8.dp),
                contentScale = ContentScale.Fit,
            )
            InfoDetails(
                name = bio.name,
                modifier = Modifier.constrainAs(info) {
                    top.linkTo(image.bottom)
                }
            )
        }
    }
}

@Composable
fun InfoDetails(
    name: String,
    modifier: Modifier = Modifier
) {
    Column(modifier = modifier.padding(Dimens.PaddingLarge)) {
        Text(
            text = name,
            style = MaterialTheme.typography.displaySmall,
            modifier = Modifier
                .padding(
                    start = Dimens.PaddingSmall,
                    end = Dimens.PaddingSmall,
                    bottom = Dimens.PaddingNormal
                )
                .align(Alignment.CenterHorizontally)
        )
        Box(
            Modifier
                .align(Alignment.CenterHorizontally)
                .padding(
                    start = Dimens.PaddingSmall,
                    end = Dimens.PaddingSmall,
                    bottom = Dimens.PaddingNormal
                )
        ) {
            Text(text = "description")
        }
    }
}
