package com.menagerie.ophelia.view.components

import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.text.KeyboardActions
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalFocusManager
import androidx.compose.ui.unit.dp
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewmodel.compose.viewModel
import com.menagerie.ophelia.database.polycule.entity.Bio
import com.menagerie.ophelia.database.polycule.entity.viewmodel.BioListViewModel

@Composable
fun InputFieldComponent(
    text: String,
    onChange: (String) -> Unit,
    modifier: Modifier = Modifier,
    singleLine: Boolean = true,
    label: String = "Some val",
    placeHolder: String = "some place",
    keyboardActions: KeyboardActions = KeyboardActions.Default,
) {
    OutlinedTextField(
        value = text,
        onValueChange = onChange,
        label = { Text(text = label)},
        placeholder = { Text(text = placeHolder)},
        modifier = modifier,
        singleLine = singleLine,
        )
}


@Composable
fun InputFieldState(
    value: String,
    label: String,
    placeholder: String,
    modifier: Modifier,
    onValChange: ((String) -> Unit)?
) {

    Column(
        modifier = Modifier
    ) {
        InputField(value, label, placeholder, modifier) { onValChange }
        Spacer(modifier = Modifier.padding(10.dp))
    }
}

@Composable
fun InputField(
    text: String,
    label: String,
    placeholder: String,
    modifier: Modifier,
    onValChange: ((String) -> Unit)?
) {
    val focusManager = LocalFocusManager.current

    if (onValChange != null) {
        InputFieldComponent(
            text = text,
            onChange = onValChange,
            label = label,
            placeHolder = placeholder,
            modifier = modifier,
            keyboardActions = KeyboardActions(onDone = { focusManager.clearFocus() })
        )
    }
}

class InputViewModel : ViewModel() {
    private var _bio: MutableLiveData<Bio> = MutableLiveData(Bio())
    var bio: LiveData<Bio> = _bio
    fun onNameChange(name: String) {
        _bio.value?.name ?: name
    }
}