package com.menagerie.ophelia.view.newUser

import android.annotation.SuppressLint
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.text.KeyboardActions
import androidx.compose.material3.Button
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.LinearProgressIndicator
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.livedata.observeAsState
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalFocusManager
import androidx.compose.ui.unit.dp
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewmodel.compose.viewModel
import com.menagerie.ophelia.database.polycule.entity.Bio
import com.menagerie.ophelia.database.polycule.entity.viewmodel.BioListViewModel
import com.menagerie.ophelia.view.OpheliaFace
import com.menagerie.ophelia.view.OpheliaWhatsYourName
import com.menagerie.ophelia.view.biographies.insertBioInDB
import com.menagerie.ophelia.view.components.InputFieldComponent

@SuppressLint("UnusedMaterial3ScaffoldPaddingParameter")
@Composable
fun AddNameScreen(
    onGo: (Bio) -> Unit = {},
) {
    val inputViewModel = InputViewModel()
    val mBioListViewModel: BioListViewModel = viewModel(
        factory = BioListViewModel.BioListViewModelFactory()
    )
    Scaffold {
        Column(
            modifier = Modifier.fillMaxSize()
        ) {
            val bio: Bio by inputViewModel.bio.observeAsState(Bio())

            OpheliaFace(
                modifier = Modifier
                    .align(Alignment.CenterHorizontally)
                    .padding(top = 12.dp)
            )
            LinearProgressIndicator(
                progress = 0f,
                modifier = Modifier.fillMaxWidth()
            )
            OpheliaWhatsYourName(modifier = Modifier)
            InputFieldState(
                inputViewModel,
                bio.name,
                "name",
                "Ophelia",
                modifier = Modifier
                    .width(120.dp)
            )
            Spacer(
                modifier = Modifier
                    .fillMaxSize()
                    .weight(1f)

            )
            Button(
                modifier = Modifier
                    .align(Alignment.End),
                onClick = {
                    insertBioInDB(inputViewModel.bio.value, mBioListViewModel)
                    onGo(bio)
                }
            ) {
                Text(text = "Continue")
            }
        }
    }
}

@Composable
fun InputFieldState(
    inputViewModel: InputViewModel,
    value: String,
    label: String,
    placeholder: String,
    modifier: Modifier
) {

    Column(
        modifier = Modifier
    ) {
        InputField(value, label, placeholder, modifier) { inputViewModel.onInputChange(it) }
        Spacer(modifier = Modifier.padding(10.dp))
    }
}

@Composable
fun InputField(
    text: String,
    label: String,
    placeholder: String,
    modifier: Modifier,
    onValChange: ((String) -> Unit)?
) {
    val focusManager = LocalFocusManager.current

    if (onValChange != null) {
        InputFieldComponent(
            text = text,
            onChange = onValChange,
            label = label,
            placeHolder = placeholder,
            modifier = modifier,
            keyboardActions = KeyboardActions(onDone = { focusManager.clearFocus() })
        )
    }
}

class InputViewModel : ViewModel() {
    private val _bio: MutableLiveData<Bio> = MutableLiveData(Bio())
    val bio: LiveData<Bio> = _bio
    fun onInputChange(name: String) {
        val newBio = Bio(
            name = name,
        )
        _bio.value = newBio
    }
}